@@ -5,10 +5,10 @@ import saxo
 
 @saxo.event("PRIVMSG")
 def exclamation(irc):
-    if irc.text == irc.client["nick"] + "!":
+    if irc.text == irc.config["nick"] + "!":
         irc.say(irc.nick + "!")
 
 @saxo.event("PRIVMSG")
 def prefix(irc):
-    if irc.text == irc.client["nick"] + ": prefix?":
-        irc.reply('"' + irc.client["prefix"] + '"')
+    if irc.text == irc.config["nick"] + ": prefix?":
+        irc.reply('"' + irc.config["prefix"] + '"')
