@@ -1,10 +1,35 @@
 # Copyright 2013, Sean B. Palmer
 # Source: http://inamidst.com/saxo/
 
-version = "0.1.005"
+version = "0.2.001"
 # WARNING: If updating anything before this message,
 # change the offset in setup.py
 
+# TODO list:
+# - Think about folding common.py into saxo.py
+# - Note the use of [plugins] in the config
+# - Document all of the available config variables
+# - Allow wildcards in [client].owner
+# - Implement [plugins].sync, copy or symlink as options
+# - Make .seen give nicely formatted dates
+# - Write documentation about basic bot operation
+# - Document the saxo, irc, self, and instruction interfaces
+# - Implement .join and .part, with persistent config writing
+# - Implement .visit and .part-analogue-with-visit
+# - Think about a packaging system
+# - Maybe write up a little guide to installing py3.3 with sqlite support
+# - Allow JSON submission to the socket IPC interface
+# - Survey commands to see if splitting or joining args is most common
+# - Write docstrings for all the public interfaces
+# - Return False to suppress error message from saxo.pipe?
+# - Allow multiple, possibly configurable, lines of output from commands?
+# - Track and mention the exit codes of command processes
+# - Handle events with commands set to "*"
+# - Plugins for pre_exec?
+# - Mode for accessing commands? ./saxo command example
+# - Make it possible to set periodic commands
+# - Make sure the test server exits correctly
+
 path = None
 
 if "__file__" in vars():
@@ -27,31 +52,23 @@ if "__path__" in vars():
 
         del directory
 
-# monitor (post) - event priority low?
-# doesn't really matter since they're threaded...
-# environment
+# Decorators:
+#
+# saxo.command(name)
+# saxo.event(command="*", synchronous=False)
+# saxo.pipe(function)
+# saxo.setup(function)
+#
+# Other:
+#
+# saxo.client(command, *args, base=None)
+# saxo.database(name=None)
+# saxo.request(*args, **kargs)
+# saxo.script(argv)
 
-def command(name):
-    def decorate(function):
-        @event("PRIVMSG")
-        def inner(irc):
-            prefix = irc.client["prefix"]
-            if irc.text.startswith(prefix):
-                length = len(prefix)
-                text = irc.text[length:]
+# TODO: environment modification?
 
-                if " " in text:
-                    cmd, arg = text.split(" ", 1)
-                else:
-                    cmd, arg = text, ""
-
-                if cmd == name:
-                    irc.arg = arg
-                    function(irc)
-        return inner
-    return decorate
-
-def communicate(command, args, base=None):
+def client(command, *args, base=None):
     import base64
     import os
     import pickle
@@ -69,15 +86,34 @@ def communicate(command, args, base=None):
     client.send(command + b" " + base64.b64encode(pickled) + b"\n")
     client.close()
 
-# TODO: Can't call this saxo.database, because the import sets that attribute
-def db(name=None):
+def command(name):
+    def decorate(function):
+        @event("PRIVMSG")
+        def inner(irc):
+            prefix = irc.config["prefix"]
+            if irc.text.startswith(prefix):
+                length = len(prefix)
+                text = irc.text[length:]
+
+                if " " in text:
+                    cmd, arg = text.split(" ", 1)
+                else:
+                    cmd, arg = text, ""
+
+                if cmd == name:
+                    irc.arg = arg
+                    function(irc)
+        return inner
+    return decorate
+
+def database(name=None):
     import os
 
     # Save PEP 3122!
     if "." in __name__:
-        from .database import Database
+        from .sqlite import Database
     else:
-        from database import Database
+        from sqlite import Database
 
     if name is None:
         base = os.environ["SAXO_BASE"]
@@ -93,7 +129,6 @@ def event(command="*", synchronous=False):
         return function
     return decorate
 
-# TODO: pipe is somewhat of a misnomer since it uses argv[1] now
 def pipe(function):
     # This gives you:
     # * concise code
@@ -109,16 +144,20 @@ def pipe(function):
 
     # Save PEP 3122!
     if "." in __name__:
-        from . import generic
+        from . import common
     else:
-        import generic
+        import common
 
-    generic.exit_cleanly()
-    # TODO: Read from stdin if there is no sys.argv[1]?
-    arg = sys.argv[1]
-    # We have to do this because python converts the commands to surrogates
-    # http://stackoverflow.com/a/7077803
-    arg = os.fsencode(arg).decode("utf-8")
+    common.exit_cleanly()
+
+    if len(sys.argv) > 1:
+        arg = sys.argv[1]
+        # We have to do this because python converts arguments to surrogates
+        # http://stackoverflow.com/a/7077803
+        arg = os.fsencode(arg).decode("utf-8")
+    else:
+        arg = sys.stdin.buffer.readline()
+        arg = arg.rstrip(b"\r\n")
 
     try: result = function(arg)
     except Exception as err:
